@@ -2,10 +2,16 @@
 from collections import OrderedDict
 
 import pytest
+import sys
 import yaml
 
 from apispec import APISpec, BasePlugin
-from apispec.exceptions import APISpecError, DuplicateComponentNameError
+from apispec.exceptions import (
+    APISpecError,
+    DuplicateComponentNameError,
+    DuplicateParameterError,
+    InvalidParameterError,
+)
 
 from .utils import (
     get_schemas,
@@ -60,7 +66,8 @@ def spec(request):
 
 class TestAPISpecInit:
     def test_raises_wrong_apispec_version(self):
-        with pytest.raises(APISpecError) as excinfo:
+        message = "Not a valid OpenAPI version number:"
+        with pytest.raises(APISpecError, match=message):
             APISpec(
                 "Swagger Petstore",
                 version="1.0.0",
@@ -68,7 +75,6 @@ class TestAPISpecInit:
                 info={"description": description},
                 security=[{"apiKey": []}],
             )
-        assert "Not a valid OpenAPI version number:" in str(excinfo)
 
 
 class TestMetadata:
@@ -184,6 +190,7 @@ class TestComponents:
             "type": "integer",
             "in": "path",
             "name": "PetId",
+            "required": True,
         }
 
     def test_parameter_is_chainable(self, spec):
@@ -262,10 +269,7 @@ class TestPath:
                     }
                 ],
                 "responses": {
-                    "200": {
-                        "schema": {"$ref": "#/definitions/Pet"},
-                        "description": "successful operation",
-                    },
+                    "200": {"schema": "Pet", "description": "successful operation"},
                     "400": {"description": "Invalid ID supplied"},
                     "404": {"description": "Pet not found"},
                 },
@@ -353,6 +357,51 @@ class TestPath:
         assert "get" in p
         assert "put" in p
 
+    @pytest.mark.parametrize("openapi_version", ("2.0", "3.0.0"))
+    def test_path_called_twice_with_same_operations_parameters(self, openapi_version):
+        """Test calling path twice with same operations or parameters
+
+        operations and parameters being mutated by clean_operations and plugin helpers
+        should not make path fail on second call
+        """
+
+        class TestPlugin(BasePlugin):
+            def path_helper(self, path, operations, parameters, **kwargs):
+                """Mutate operations and parameters"""
+                operations.update({"post": {"responses": {"201": "201ResponseRef"}}})
+                parameters.append("ParamRef_3")
+                return path
+
+        spec = APISpec(
+            title="Swagger Petstore",
+            version="1.0.0",
+            openapi_version=openapi_version,
+            plugins=[TestPlugin()],
+        )
+
+        path = "/pet/{petId}"
+        parameters = ["ParamRef_1"]
+        operation = {
+            "parameters": ["ParamRef_2"],
+            "responses": {"200": "200ResponseRef"},
+        }
+
+        spec.path(path=path, operations={"get": operation}, parameters=parameters)
+        spec.path(path=path, operations={"put": operation}, parameters=parameters)
+        operations = (get_paths(spec))[path]
+        assert (
+            operations["get"]
+            == operations["put"]
+            == {
+                "parameters": [build_ref(spec, "parameter", "ParamRef_2")],
+                "responses": {"200": build_ref(spec, "response", "200ResponseRef")},
+            }
+        )
+        assert operations["parameters"] == [
+            build_ref(spec, "parameter", "ParamRef_1"),
+            build_ref(spec, "parameter", "ParamRef_3"),
+        ]
+
     def test_path_ensures_path_parameters_required(self, spec):
         path = "/pet/{petId}"
         spec.path(
@@ -362,9 +411,9 @@ class TestPath:
         assert get_paths(spec)[path]["put"]["parameters"][0]["required"] is True
 
     def test_path_with_no_path_raises_error(self, spec):
-        with pytest.raises(APISpecError) as excinfo:
+        message = "Path template is not specified"
+        with pytest.raises(APISpecError, match=message):
             spec.path()
-        assert "Path template is not specified" in str(excinfo)
 
     def test_path_summary_description(self, spec):
         summary = "Operations on a Pet"
@@ -398,6 +447,85 @@ class TestPath:
                 == metadata["components"]["parameters"]["test_parameter"]
             )
 
+    @pytest.mark.parametrize(
+        "parameters",
+        ([{"name": "petId"}], [{"in": "path"}]),  # missing "in"  # missing "name"
+    )
+    def test_invalid_parameter(self, spec, parameters):
+        path = "/pet/{petId}"
+
+        with pytest.raises(InvalidParameterError):
+            spec.path(path=path, operations=dict(put={}, get={}), parameters=parameters)
+
+    def test_parameter_duplicate(self, spec):
+        spec.path(
+            path="/pet/{petId}",
+            operations={
+                "get": {
+                    "parameters": [
+                        {"name": "petId", "in": "path"},
+                        {"name": "petId", "in": "query"},
+                    ]
+                }
+            },
+        )
+
+        with pytest.raises(DuplicateParameterError):
+            spec.path(
+                path="/pet/{petId}",
+                operations={
+                    "get": {
+                        "parameters": [
+                            {"name": "petId", "in": "path"},
+                            {"name": "petId", "in": "path"},
+                        ]
+                    }
+                },
+            )
+
+    def test_global_parameters(self, spec):
+        path = "/pet/{petId}"
+        route_spec = self.paths["/pet/{petId}"]["get"]
+
+        spec.components.parameter("test_parameter", "path", route_spec["parameters"][0])
+        spec.path(
+            path=path,
+            operations=dict(put={}, get={}),
+            parameters=[{"name": "petId", "in": "path"}, "test_parameter"],
+        )
+
+        assert get_paths(spec)[path]["parameters"] == [
+            {"name": "petId", "in": "path", "required": True},
+            build_ref(spec, "parameter", "test_parameter"),
+        ]
+
+    def test_global_parameter_duplicate(self, spec):
+        path = "/pet/{petId}"
+        spec.path(
+            path=path,
+            operations=dict(put={}, get={}),
+            parameters=[
+                {"name": "petId", "in": "path"},
+                {"name": "petId", "in": "query"},
+            ],
+        )
+
+        assert get_paths(spec)[path]["parameters"] == [
+            {"name": "petId", "in": "path", "required": True},
+            {"name": "petId", "in": "query"},
+        ]
+
+        with pytest.raises(DuplicateParameterError):
+            spec.path(
+                path=path,
+                operations=dict(put={}, get={}),
+                parameters=[
+                    {"name": "petId", "in": "path"},
+                    {"name": "petId", "in": "path"},
+                    "test_parameter",
+                ],
+            )
+
     def test_response(self, spec):
         route_spec = self.paths["/pet/{petId}"]["get"]
 
@@ -422,12 +550,40 @@ class TestPath:
                 == metadata["components"]["responses"]["test_response"]
             )
 
+    @pytest.mark.skipif(
+        int(sys.version[0]) == 2, reason="HTTPStatus only available in Python3"
+    )
+    def test_response_with_HTTPStatus_code(self, spec):
+        from http import HTTPStatus
+
+        code = HTTPStatus(200)
+        spec.path(
+            path="/pet/{petId}",
+            operations={"get": {"responses": {code: "test_response"}}},
+        )
+
+        assert "200" in get_paths(spec)["/pet/{petId}"]["get"]["responses"]
+
+    def test_response_with_status_code_range(self, spec, recwarn):
+        status_code = "2XX"
+
+        spec.path(
+            path="/pet/{petId}",
+            operations={"get": {"responses": {status_code: "test_response"}}},
+        )
+
+        if spec.openapi_version.major < 3:
+            assert len(recwarn) == 1
+            assert recwarn.pop(UserWarning)
+
+        assert status_code in get_paths(spec)["/pet/{petId}"]["get"]["responses"]
+
     def test_path_check_invalid_http_method(self, spec):
         spec.path("/pet/{petId}", operations={"get": {}})
         spec.path("/pet/{petId}", operations={"x-dummy": {}})
-        with pytest.raises(APISpecError) as excinfo:
+        message = "One or more HTTP methods are invalid"
+        with pytest.raises(APISpecError, match=message):
             spec.path("/pet/{petId}", operations={"dummy": {}})
-        assert "One or more HTTP methods are invalid" in str(excinfo)
 
 
 class TestPlugins:
@@ -446,10 +602,11 @@ class TestPlugins:
                 if not return_none:
                     return {"description": "42"}
 
-            def path_helper(self, path, operations, **kwargs):
+            def path_helper(self, path, operations, parameters, **kwargs):
                 if not return_none:
                     if path == "/path_1":
                         operations.update({"get": {"responses": {"200": {}}}})
+                        parameters.append({"name": "page", "in": "query"})
                         return "/path_1_modified"
 
             def operation_helper(self, path, operations, **kwargs):
@@ -525,7 +682,10 @@ class TestPlugins:
         if return_none:
             assert paths["/path_1"] == {}
         else:
-            assert paths["/path_1_modified"] == {"get": {"responses": {"200": {}}}}
+            assert paths["/path_1_modified"] == {
+                "get": {"responses": {"200": {}}},
+                "parameters": [{"in": "query", "name": "page"}],
+            }
 
     @pytest.mark.parametrize("openapi_version", ("2.0", "3.0.0"))
     def test_plugin_operation_helper_is_used(self, openapi_version):
