@@ -51,7 +51,8 @@ class TestDefinitionHelper:
             openapi_version="2.0",
             plugins=(MarshmallowPlugin(schema_name_resolver=resolver),),
         )
-        assert {} == get_schemas(spec)
+        with pytest.raises(KeyError):
+            get_schemas(spec)
 
         spec.components.schema("analysis", schema=schema)
         spec.path(
@@ -84,7 +85,8 @@ class TestDefinitionHelper:
             openapi_version="2.0",
             plugins=(MarshmallowPlugin(schema_name_resolver=resolver),),
         )
-        assert {} == get_schemas(spec)
+        with pytest.raises(KeyError):
+            get_schemas(spec)
 
         spec.components.schema("analysis", schema=schema)
         spec.path(
@@ -115,7 +117,8 @@ class TestDefinitionHelper:
             openapi_version="2.0",
             plugins=(MarshmallowPlugin(schema_name_resolver=resolver),),
         )
-        assert {} == get_schemas(spec)
+        with pytest.raises(KeyError):
+            get_schemas(spec)
 
         with pytest.raises(
             APISpecError, match="Name resolver returned None for schema"
@@ -242,6 +245,13 @@ class TestComponentResponseHelper:
         assert resolved_schema["properties"]["name"]["type"] == "string"
         assert resolved_schema["properties"]["password"]["type"] == "string"
 
+    @pytest.mark.parametrize("spec", ("3.0.0",), indirect=True)
+    def test_content_without_schema(self, spec):
+        resp = {"content": {"application/json": {"example": {"name": "Example"}}}}
+        spec.components.response("GetPetOk", resp)
+        response = get_responses(spec)["GetPetOk"]
+        assert response == resp
+
 
 class TestCustomField:
     def test_can_use_custom_field_decorator(self, spec_fixture):
@@ -562,7 +572,11 @@ class TestOperationHelper:
             operations={
                 "get": {
                     "parameters": [
-                        {"in": "body", "schema": {"type": "array", "items": PetSchema}}
+                        {
+                            "name": "petSchema",
+                            "in": "body",
+                            "schema": {"type": "array", "items": PetSchema},
+                        }
                     ],
                     "responses": {
                         200: {"schema": {"type": "array", "items": PetSchema}}
@@ -588,7 +602,8 @@ class TestOperationHelper:
                 "get": {
                     "parameters": [
                         {
-                            "in": "body",
+                            "name": "Pet",
+                            "in": "query",
                             "content": {
                                 "application/json": {
                                     "schema": {"type": "array", "items": PetSchema}
